# -*- coding: utf-8 -*-
"""
    slixmpp.util.sasl
    ~~~~~~~~~~~~~~~~~~~

    This module was originally based on Dave Cridland's Suelta library.

    Part of Slixmpp: The Slick XMPP Library

    :copryight: (c) 2004-2013 David Alan Cridland
    :copyright: (c) 2013 Nathanael C. Fritz, Lance J.T. Stout

    :license: MIT, see LICENSE for more details
"""

from slixmpp.util.sasl.client import *
from slixmpp.util.sasl.mechanisms import *
